function [medianpred,lowerbound,upperbound,PredVarLabels]=PredSummary(Data,Preds,FutureData,PlotPred,CoverProb,HPD,FirstDateOfData,season,Horizon,LengthOfPast,Variables,PlotVariables,VarLabels,GridOn,DottedPred,AllVarsInOneFig,GraphTitle)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PURPOSE:      Summarizes draws from the predictive density to a median forecast and predictive uncertainty bands. Plots fan charts.
%
% INPUT:        Data             (NObs-by-NVars)            Data matrix
%               Preds            (NIter-by-NVars-NHorizons) 3-dim matrix of draw from predictive distr.     
%               FutureData       (N-by-NVars)
%               PlotPred         (scalar)                   If PlotPred=1, fan charts are graphed.                       
%               CoverProb        (1-by-NCoverProb)          Vector of up to three probability contents of the pred. intervals. Ex. CoverProb=[0.68 0.95]                      
%               HPD              (scalar)                   If HPD=1, intervals are HPD, otherwise symmetric intervals. HPD is time-consuming.
%               FirstDateOfData  (string)                   Date of first data point in Data. Ex. '1970Q1'.
%               Horizon          (scalar)                   Determines the forecast horizon in the graphs.
%               LengthOfPast     (scalar)                   Determines the number of historical observations to plot before the predictions.
%               Variables        (1-by-NPredVars)           Vector of variables for which predictions are computed indices relative to Preds.  
%               PlotVariables    (1-by-NPlotVars)           Vector of variable indicies relative relative to Preds.
%               VarLabels        (NVars-cell)               Variable labels.
%               GridOn           (scalar)                   If GridOn=1, grid lines are plotted in the fan chart.
%               DottedPred       (scalar)                   If DottedPred=1, dots are plotted for each forecast point.
%
% OUTPUT:       medianpred
%               lowerbound
%               upperbound
%               PredVarLabels
%
% AUTHOR:       Mattias Villani, Research Department, Sveriges Riksbank and
%               Department of Statistics, Stockholm University. 
%               E-mail: mattias.villani@riksbank.se
%
% FIRST VER:    2004-10-17
% REVISED:      2007-09-09
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  USER INPUTS  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Grayness of uncertainty bands
colorband=[0.65 0.65 0.65;0.459 0.456 1.000;0.071 0.110 0.553]; % Blue/Purple-ish color
% Handling of the time-axis
linewidthpred=1;
LineWidthRealized=1.5;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Making PlotVariables relative to Variables
PlotVariables=find(ismember(Variables,PlotVariables));
nPlotVars=length(PlotVariables);

if nargin==13
    AllVarsInOneFig=1;
    GraphTitle='';
end
if nargin==14
    GraphTitle='';
end

if length(CoverProb)>3 % Automatic colors if more than three band probabilities
    colorband=cool(length(CoverProb));
end

MaxNFuture=min(size(FutureData,1),Horizon);
if MaxNFuture>0
    FutureData=FutureData(1:MaxNFuture,:);
end
nvars=length(Variables);
niter=size(Preds,1);
[T,p]=size(Data);
AllRealizedObs=[Data;FutureData];
[TExtended,p]=size(AllRealizedObs); % TExtended is the number of observations before standing period + number of predictions

CoverProb=CoverProb(:); % Force to be a column vector
CoverProb=flipud(sort(CoverProb)); % Arrange in descending order
ncoverprobs=length(CoverProb);
low=round(((1-CoverProb)/2)*niter);
high=round(niter-((1-CoverProb)/2)*niter);
HPDerror=[];
colorband=colorband(1:ncoverprobs,:);

if LengthOfPast+Horizon>50 % Series are too long to be plotted 'optimal' subplots.
    nrows=nPlotVars;
    ncols=1;
else
    [nrows,ncols]=ConstructOptimalSubplot(nPlotVars);
end

% Computing all date vectors
AllRealizedDates=FirstDateOfData:(1/season):FirstDateOfData+(1/season)*(TExtended-1); % Dates for all realized observations
HistAndForecastDates=FirstDateOfData:(1/season):FirstDateOfData+(1/season)*(T+Horizon-1); % Dates for historical data and forecast horizon
ForecastDates=HistAndForecastDates(end-Horizon+1:end); % Dates for the forecasting period
HistDates=setdiff(HistAndForecastDates,ForecastDates); % Dates for historical data
RecentHistDates=HistDates(end-LengthOfPast+1:end); % Dates for most recent historical data
ForecastDatesPlusLast=[HistDates(end) ForecastDates];

% Initialization
medianpred=zeros(Horizon,nvars);
lowerbound=zeros(Horizon,nvars,ncoverprobs);
upperbound=zeros(Horizon,nvars,ncoverprobs);


% Computing the median prediction and uncertainty bands
if niter>1
    for vars=Variables
        for h=1:Horizon
            medianpred(h,vars)=median(Preds(:,vars,h));
            s=sort(Preds(:,vars,h));
            for v=1:ncoverprobs
                if HPD==1 % Highest posterior density (HPD) prediction bands
                    [hpd,hpdregion,actualcoverage]=HPDregions(s,CoverProb(v));
                    if size(hpd,1)>1
                        HPDerror=[HPDerror;vars h v];
                        hpd(1)=s(low(v));
                        hpd(2)=s(high(v));
                    end
                    lowerbound(h,vars,v)=hpd(1);
                    upperbound(h,vars,v)=hpd(2);
                else % Symmetric prediction bands.
                    lowerbound(h,vars,v)=s(low(v));
                    upperbound(h,vars,v)=s(high(v));
                end
            end
        end
    end
else
    for vars=Variables
        for h=1:Horizon
            medianpred(h,vars)=Preds(1,vars,h)';
            lowerbound=[];
            upperbound=[];
        end
    end
end

%%%%%%% Plotting the actual data and predictions %%%%%%%%
if PlotPred==1
    k=0;
    for var=PlotVariables
        k=k+1;
        if AllVarsInOneFig==1
            if k==1
                FigHandle=figure;
                set(FigHandle,'name',GraphTitle)
            end
            subplot(nrows,ncols,k)
        else
            FigHandle=figure;
            set(FigHandle,'name',GraphTitle)
        end
        
        hold on;
        
        % Plotting actual data
        handle1=plot(AllRealizedDates(T-LengthOfPast+1:TExtended),AllRealizedObs(T-LengthOfPast+1:TExtended,var),'k-');
        set(handle1,'linewidth',LineWidthRealized);

        % Plotting uncertainty bands (fan charts)
        if niter>1
            for v=1:ncoverprobs
                y=[Data(T,var);upperbound(:,var,v);flipud(lowerbound(:,var,v));Data(T,var)];
                x=[ForecastDatesPlusLast fliplr(ForecastDatesPlusLast)]';
                a1=patch(x,y,colorband(v,:));
                set(a1,'faceLighting','phong')
                set(a1,'facealpha',0.5)
                set(a1,'edgecolor',min([1.05*colorband(v,:)],[1 1 1]));
                set(a1,'edgealpha',0.5)
            end 
        end
        
        % Plotting median forecasts
        if DottedPred==0
            handle2=plot(ForecastDatesPlusLast,[Data(T,var);medianpred(:,var)],'k-');
        else
            handle2=plot(ForecastDatesPlusLast,[Data(T,var);medianpred(:,var)],'b.-');
        end
        set(handle2,'linewidth',linewidthpred);

        % Overriding Matlab's automatic axis limits
        if ncoverprobs>0
            minimum=min([AllRealizedObs(T-LengthOfPast+1:TExtended,var)' lowerbound(:,var,1)']);
            maximum=max([AllRealizedObs(T-LengthOfPast+1:TExtended,var)' upperbound(:,var,1)']);
        else
            minimum=min([AllRealizedObs(T-LengthOfPast+1:TExtended,var)' medianpred(:,var)']);
            maximum=max([AllRealizedObs(T-LengthOfPast+1:TExtended,var)' medianpred(:,var)']);
        end
        if minimum>0
            minimum=0.85*minimum;
        else
            minimum=1.15*minimum;
        end
        if maximum>0
            maximum=1.15*maximum;
        else
            maximum=0.85*maximum;
        end
        
        % Setting axis properties
        axis tight
        CurrentAxis=axis;
        axis([CurrentAxis(1:2) minimum maximum]);
        title(VarLabels(var))
        box off
        set(gca,'fontsize',8);
        if GridOn==1
            grid on
        else
            %set(gca,'xtick',[])
        end

    end
end

% HPD intervals may be disconnected. The following codes prints lists
% all disconnected intervals which were replace by symmetric intervals. 
PredVarLabels=VarLabels(Variables);
if isempty(HPDerror)==0
    disp('')
    disp('WARNING: The following HPD intervals were disconnected and replaced by symmetric intervals:')
    disp('')
    for i=1:size(HPDerror,1)
        disp('')
        disp('')
        disp('')
        disp(VarLabels(HPDerror(i,1)))
        disp('    Horizon   Interval Coverage')
        fprintf(1,'%8.0f  %8.0f\n\n',[HPDerror(i,2) CoverProb(HPDerror(i,3))])
    end
end